/************************************************************************/
/*																		*/
/*	PmodRF1.h	--  PmodRF1 Wireless Transceiver Interface              */
/*																		*/
/************************************************************************/
/*	Author: Michael T. Alexander										*/
/*	Copyright 2009, Digilent Inc.										*/
/************************************************************************/
/*  File Description:													*/
/*																		*/
/*  This module contains declarations and definitions for interfacing   */
/*  the wireless transceiver used on the PmodRF1.                       */
/*																		*/
/************************************************************************/
/*  Revision History:													*/
/*																		*/
/*  05/22/2009 (MichaelA): created                                      */
/*																		*/
/************************************************************************/

#if !defined(_PMODRF1_INC)
#define _PMODRF1_INC

#include "config.h"
#include "stdtypes.h"

/* ------------------------------------------------------------ */
/*					    Register Addresses  					*/
/* ------------------------------------------------------------ */

#define     rfrTrxStatus            0x01
#define     rfrTrxState             0x02
#define     rfrTrxCtrl0             0x03
#define     rfrTrxCtrl1             0x04

#define     rfrPhyTxPwr             0x05
#define     rfrPhyRssi              0x06
#define     rfrPhyEdLevel           0x07
#define     rfrPhyCcCca             0x08
#define     rfrCcaThres             0x09

#define     rfrRxCtrl               0x0a
#define     rfrSfdValue             0x0b
#define     rfrTrxCtrl2             0x0c
#define     rfrAntDiv               0x0d
#define     rfrIrqMask              0x0e
#define     rfrIrqStatus            0x0f

#define     rfrVreqCtrl             0x10
#define     rfrBatmon               0x11
#define     rfrXoscCtrl             0x12
#define     rfrCcCtrl0              0x13
#define     rfrCcCtrl1              0x14

#define     rfrRxSyn                0x15
#define     rfrRfCtrl0              0x16
#define     rfrXahCtrl1             0x17
#define     rfrFtnCtrl              0x18
#define     rfrRfCtrl1              0x19

#define     rfrPllCf                0x1a
#define     rfrPllDcu               0x1b
#define     rfrPartNum              0x1c
#define     rfrVersionNum           0x1d
#define     rfrManId0               0x1e
#define     rfrManId1               0x1f

#define     rfrShortAddr0           0x20
#define     rfrShortAddr1           0x21
#define     rfrPanId0               0x22
#define     rfrPanId1               0x23

#define     rfrIeeeAddr0            0x24
#define     rfrIeeeAddr1            0x25
#define     rfrIeeeAddr2            0x26
#define     rfrIeeeAddr3            0x27
#define     rfrIeeeAddr4            0x28
#define     rfrIeeeAddr5            0x29
#define     rfrIeeeAddr6            0x2a
#define     rfrIeeeAddr7            0x2b

#define     rfrXahCtrl0             0x2c
#define     rfrCsmaSeed0            0x2d
#define     rfrCsmaSeed1            0x2e
#define     rfrCsmaBe               0x2f

/* ------------------------------------------------------------ */
/*					      Register Fields     					*/
/* ------------------------------------------------------------ */

/*  TRX_STATUS
*/
#define     bnCcaDone               7
#define     bnCcaStatus             6

#define     bnTrxStatus4            4
#define     bnTrxStatus3            3
#define     bnTrxStatus2            2
#define     bnTrxStatus1            1
#define     bnTrxStatus0            0

/*  TRX_STATE
*/
#define     bnTracStatus2           7
#define     bnTracStatus1           6
#define     bnTracStatus0           5
#define     bnTrxCmd4               4
#define     bnTrxCmd3               3
#define     bnTrxCmd2               2
#define     bnTrxCmd1               1
#define     bnTrxCmd0               0

/*  TRX_CTRL_0
*/
#define     bnPadIo1                7
#define     bnPadIo0                6
#define     bnPadIoClkm1            5
#define     bnPadIoClkm0            4
#define     bnClkmShaSel            3
#define     bnClkmCtrl2             2
#define     bnClkmCtrl1             1
#define     bnClkmCtrl0             0

/*  TRX_CTRL_1
*/
#define     bnPaExtEn               7
#define     bnIrq2ExtEn             6
#define     bnTxAutoCrcOn           5
#define     bnRxBlCtrl              4
#define     bnSpiCmdMode1           3
#define     bnSpiCmdMode0           2
#define     bnIrqMaskMode           1
#define     bnIrqPolarity           0

/*  PHY_TX_PWR
*/
#define     bnPaBoost               7
#define     bnGcPa1                 6
#define     bnGcPa0                 5
#define     bnTxPwr4                4
#define     bnTxPwr3                3
#define     bnTxPwr2                2
#define     bnTxPwr1                1
#define     bnTxPwr0                0

/*  PHY_RSSI
*/
#define     bnRxCrcValid            7
#define     bnRndValue1             6
#define     bnRndValue0             5
#define     bnRssi4                 4
#define     bnRssi3                 3
#define     bnRssi2                 2
#define     bnRssi1                 1
#define     bnRssi0                 0

/*  PHY_CC_CCA
*/
#define     bnCcaRequest            7
#define     bnCcaMode1              6
#define     bnCcaMode0              5
#define     bnChannel4              4
#define     bnChannel3              3
#define     bnChannel2              2
#define     bnChannel1              1
#define     bnChannel0              0

/*  CCA_THRES
*/
#define     bnCcaEdThres3           3
#define     bnCcaEdThres2           2
#define     bnCcaEdThres1           1
#define     bnCcaEdThres0           0

/*  RX_CTRL
*/
#define     bnJcmEn                 5

/*  TRX_CTRL_2
*/
#define     bnRxSafeMode            7
#define     bnTrxOffAvddEn          6
#define     bnOqpskScramEn          5
#define     bnOqpskSub1RcEn         4
#define     bnBpskOqpsk             3
#define     bnSubMode               2
#define     bnOqpskDataRate1        1
#define     bnOqpskDataRate0        0

/*  ANT_DIV
*/
#define     bnAntExtSwEn            2
#define     bnAntCtrl1              1
#define     bnAntCtrl0              0

/*  IRQ_MASK 
*/
#define     bnMaskBatLow            7
#define     bnMaskTrxUr             6
#define     bnMaskAmi               5
#define     bnMaskCcaEdDone         4
#define     bnMaskTrxEnd            3
#define     bnMaskRxStart           2
#define     bnMaskPllUnlock         1
#define     bnMaskPllLock           0

/*  IRQ_STATUS
*/
#define     bnBatLow                7
#define     bnTrxUr                 6
#define     bnAmi                   5
#define     bnCcaEdDone             4
#define     bnTrxEnd                3
#define     bnRxStart               2
#define     bnPllUnlock             1
#define     bnPllLock               0

/*  VREG_CTRL
*/
#define     bnAvregExt              7
#define     bnAvddOk                6
#define     bnDvregExt              3
#define     bnDvddOk                2

/*  BATMON
*/
#define     bnBatmonOk              5
#define     bnBatmonHr              4
#define     bnBatmonVth3            3
#define     bnBatmonVth2            2
#define     bnBatmonVth1            1
#define     bnBatmonVth0            0

/*  XOSC_CTRL
*/
#define     bnXtalMode3             7
#define     bnXtalMode2             6
#define     bnXtalMode1             5
#define     bnXtalMode0             4
#define     bnXtalTrim3             3
#define     bnXtalTrim2             2
#define     bnXtalTrim1             1
#define     bnXtalTrim0             0

/*  CC_CTRL_0
*/
#define     bnCcNumber7             7
#define     bnCcNumber6             6
#define     bnCcNumber5             5
#define     bnCcNumber4             4
#define     bnCcNumber3             3
#define     bnCcNumber2             2
#define     bnCcNumber1             1
#define     bnCcNumber0             0

/*  CC_CTRL_1
*/
#define     bnCcBand2               2
#define     bnCcBand1               1
#define     bnCcBand0               0

/*  RX_SYN
*/
#define     bnRxPdtDis              7
#define     bnRxPdtLevel3           3
#define     bnRxPdtLevel2           2
#define     bnRxPdtLevel1           1
#define     bnRxPdtLevel0           0

/*  RF_CTRL_0
*/
#define     bnPaLt1                 7
#define     bnPaLt0                 6
#define     bnGcTxOffs1             1
#define     bnGcTxOffs0             0

/*  XAH_CTRL_1
*/
#define     bnCsmaLbtMode           6
#define     bnAackFltrResFt         5
#define     bnAackUpldResFt         4
#define     bnAackAckTime           2
#define     bnAackPromMode          1

/*  FTN_CTRL
*/
#define     bnFtnStart              7

/*  RF_CTRL_1
*/
#define     bnRfMc3                 7
#define     bnRfMc2                 6
#define     bnRfMc1                 5
#define     bnRfMc0                 4

/*  PLL_CF
*/
#define     bnPllCfStart            7
#define     bnPllCf4                4
#define     bnPllCf3                3
#define     bnPllCf2                2
#define     bnPllCf1                1
#define     bnPllCf0                0

/*  PLL_DCU
*/
#define     bnPllDcuStart           7

/*  XAH_CTRL_0
*/
#define     bnMaxFrameRetries3      7
#define     bnMaxFrameRetries2      6
#define     bnMaxFrameRetries1      5
#define     bnMaxFrameRetries0      4
#define     bnMaxCsmaRetries2       3
#define     bnMaxCsmaRetries1       2
#define     bnMaxCsmaRetries0       1
#define     bnSlottedOperation      0

/*  CSMA_SEED_1
*/
#define     bnAackFvnMode1          7
#define     bnAackFvnMode0          6
#define     bnAackSetPd             5
#define     bnAackDisAck            4
#define     bnAackIAmCoord          3
#define     bnCsmaSeed12            2
#define     bnCsmaSeed11            1
#define     bnCsmaSeed10            0

/*  CSMA_BE
*/
#define     bnMaxBe3                7
#define     bnMaxBe2                6
#define     bnMaxBe1                5
#define     bnMaxBe0                4
#define     bnMinBe3                3
#define     bnMinBe2                2
#define     bnMinBe1                1
#define     bnMinBe0                0

/* ------------------------------------------------------------ */
/*					      Register Masks     					*/
/* ------------------------------------------------------------ */

#define     mskTrxStatus            0x1F

#define     mskTracStatus           0xe0

#define     mskChan                 0x1F


/* ------------------------------------------------------------ */
/*					      Commands           					*/
/* ------------------------------------------------------------ */

#define     rfcRegRead              0x80
#define     rfcRegWrite             0xc0
#define     rfcFrameRead            0x20
#define     rfcFrameWrite           0x60

/*  TRX_CMD
*/
#define     rfcNop                  0x00
#define     rfcTxStart              0x02
#define     rfcForceTrxOff          0x03
#define     rfcForcePllOn           0x04
#define     rfcRxOn                 0x06
#define     rfcTrxOff               0x08
#define     rfcPllOn                0x09
#define     rfcRxAackOn             0x16
#define     rfcTxAretOn             0x19

/* ------------------------------------------------------------ */
/*					         States          					*/
/* ------------------------------------------------------------ */

#define     rfstPOn                         0x00
#define     rfstBusyRx                      0x01
#define     rfstBusyTx                      0x02
#define     rfstRxOn                        0x06
#define     rfstTrxOff                      0x08
#define     rfstPllOn                       0x09
#define     rfstSleep                       0x0f
#define     rfstBusyRxAack                  0x11
#define     rfstBusyTxAret                  0x12
#define     rfstRxAackOn                    0x16
#define     rfstTxAretOn                    0x19
#define     rfstRxOnNoclk                   0x1c
#define     rfstRxAackOnNoclk               0x1d
#define     rfstBusyRxAackNoclk             0x1e
#define     rfstStateTransitionInProgress   0x1f

/* ------------------------------------------------------------ */
/*					General Type Declarations					*/
/* ------------------------------------------------------------ */

#define rfadrBcast  0xFF

/* ------------------------------------------------------------ */
/*					Variable Declarations						*/
/* ------------------------------------------------------------ */

BYTE    rfadrMac;
BYTE    rfrLqi;

/* ------------------------------------------------------------ */
/*					        Macros		        				*/
/* ------------------------------------------------------------ */




/* ------------------------------------------------------------ */
/*					Procedure Declarations						*/
/* ------------------------------------------------------------ */

void    RF1Init( void );
BYTE    BRF1ReadReg( BYTE rfr );
void    RF1WriteReg( BYTE rfr, BYTE bVal );
BYTE    BRF1ReadFrame( BYTE* pbBuff );
void    RF1WriteFrame( BYTE* pbBuff, BYTE cbBuff );

/*  Higher level routines
*/
BYTE    BRF1ReadPkt( BYTE* prfadrSrc, BYTE* pbPkt );
void    RF1WritePkt( BYTE rfadrDest, BYTE* pbPkt, BYTE cbPkt );
void    RF1SetMac( BYTE rfadrSet );


/* ------------------------------------------------------------ */

#endif

/************************************************************************/
